unit rzipAbbrevia_interface;
{<
  Provides ZIP functions for library model or for rzip_interface unit.
  Uses Abbrevia package.

  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(May 2016)
  @lastmod(May 2016)
}

interface

uses
  SysUtils,
  Classes,
  AbZipKit,
  AbArcTyp,
  AbUtils,
  AbConst,
  AbResString;

type
  TrAbZipObject = class
    AbZip: TAbZipKit;
    ErrCode: integer;
    ErrMessage: string;
    LastFile: string;
    procedure ErrReset(FileName: string = '');
    procedure AbZipErr(Sender: TObject; Item: TAbArchiveItem; ProcessType: TAbProcessType; ErrorClass: TAbErrorClass; ErrorCode: integer);
    function OpenArchive(ZipFile: string): integer;
    function CloseArchive(Save: boolean): integer;
    function GetFileList(var ListText: string): integer;
    function ExtractAllFiles(OutputPath: string): integer;
    function ExtractFile(ExtractFile, OutputFile: string): integer;
    function ExtractToBuffer(ExtractFile: string; Buffer: Pointer; var Length: Int64): integer;
    function ExtractContent(ExtractFile: string; var Content: PChar): integer;
    function AddFile(NewFile, InputFile: string): integer;
    function AddFromBuffer(NewFile: string; Buffer: Pointer; Length: Int64): integer;
    function AddContent(NewFile: string; Content: PChar): integer;
    function GetLastErrorMsg: string;
  end;

var
  rAbZipObject: TrAbZipObject;

procedure rAbZipObject_Create;
procedure eAbZipObject_Free;

implementation

procedure TrAbZipObject.ErrReset(FileName: string = '');
begin
  ErrCode:=0;
  LastFile:=FileName;
end;

procedure TrAbZipObject.AbZipErr(Sender: TObject; Item: TAbArchiveItem; ProcessType: TAbProcessType; ErrorClass: TAbErrorClass; ErrorCode: integer);
begin
  ErrCode:=ErrorCode;
  ErrMessage:=AbStrRes(ErrCode);
  LastFile:=Item.FileName;
end;

function TrAbZipObject.OpenArchive(ZipFile: string): integer;
begin
  rAbZipObject.ErrReset(ZipFile);
  try
    rAbZipObject.AbZip.OpenArchive(ZipFile);
  except
    rAbZipObject.ErrCode:=AbErrZipInvalid;
  end;
  Result:=rAbZipObject.ErrCode;
end;

function TrAbZipObject.CloseArchive(Save: boolean): integer;
begin
  rAbZipObject.ErrReset;
  if Save then
    try
      rAbZipObject.AbZip.Save;
    except
      rAbZipObject.ErrCode:=AbStreamFull;
    end;
  rAbZipObject.AbZip.CloseArchive;
  Result:=rAbZipObject.ErrCode;
end;

function TrAbZipObject.GetFileList(var ListText: string): integer;
var
  i: integer;
  L: TStrings;
begin
  rAbZipObject.ErrReset;
  L:=TStringList.Create;
  try
    L.Clear;
    for i:=0 to rAbZipObject.AbZip.Count - 1 do
      L.Add(rAbZipObject.AbZip.Items[i].FileName);
    ListText:=PChar(L.Text);
  finally
    L.Free;
  end;
  Result:=rAbZipObject.ErrCode;
end;

function TrAbZipObject.ExtractAllFiles(OutputPath: string): integer;
begin
  rAbZipObject.ErrReset;
  rAbZipObject.AbZip.BaseDirectory:=OutputPath;
  try
    rAbZipObject.AbZip.ExtractFiles('*.*');
  except
    rAbZipObject.ErrCode:=AbReadError;
  end;
  Result:=rAbZipObject.ErrCode;
end;

function TrAbZipObject.ExtractFile(ExtractFile, OutputFile: string): integer;
var
  mStream: TFileStream;
begin
  rAbZipObject.ErrReset(ExtractFile);
  if rAbZipObject.AbZip.FindFile(ExtractFile) < 0 then
    rAbZipObject.ErrCode:=AbFileNotFound
  else
  begin
    mStream:=TFileStream.Create(OutputFile, fmCreate);
    try
      rAbZipObject.AbZip.ExtractToStream(ExtractFile, mStream);
    finally
      mStream.Free;
    end;
  end;
  Result:=rAbZipObject.ErrCode;
end;

function TrAbZipObject.ExtractToBuffer(ExtractFile: string; Buffer: Pointer; var Length: Int64): integer;
var
  mStream: TMemoryStream;
begin
  rAbZipObject.ErrReset(ExtractFile);
  if rAbZipObject.AbZip.FindFile(ExtractFile) < 0 then
    rAbZipObject.ErrCode:=AbFileNotFound
  else
  begin
    mStream:=TMemoryStream.Create;
    try
      rAbZipObject.AbZip.ExtractToStream(ExtractFile, mStream);
      if (Length <> mStream.Size) then
        Length:=mStream.Size
      else
      begin
        mStream.Position:=0;
        mStream.Read(Buffer^, mStream.Size);
      end;
    finally
      mStream.Free;
    end;
  end;
  Result:=rAbZipObject.ErrCode;
end;

function TrAbZipObject.ExtractContent(ExtractFile: string; var Content: PChar): integer;
var
  mStream: TMemoryStream;
  S: TStrings;
begin
  rAbZipObject.ErrReset(ExtractFile);
  if rAbZipObject.AbZip.FindFile(ExtractFile) < 0 then
    rAbZipObject.ErrCode:=AbFileNotFound
  else
  begin
    mStream:=TMemoryStream.Create;
    S:=TStringList.Create;
    try
      rAbZipObject.AbZip.ExtractToStream(ExtractFile, mStream);
      mStream.Position:=0;
      S.LoadFromStream(mStream);
      Content:=PChar(S.Text);
    finally
      S.Free;
      mStream.Free;
    end;
  end;
  Result:=rAbZipObject.ErrCode;
end;

function TrAbZipObject.AddFile(NewFile, InputFile: string): integer;
var
  mStream: TFileStream;
begin
  rAbZipObject.ErrReset(InputFile);
  mStream:=TFileStream.Create(InputFile, fmShareDenyNone);
  try
    rAbZipObject.AbZip.AddFromStream(NewFile, mStream);
  finally
    mStream.Free;
  end;
  Result:=rAbZipObject.ErrCode;
end;

function TrAbZipObject.AddFromBuffer(NewFile: string; Buffer: Pointer; Length: Int64): integer;
var
  mStream: TMemoryStream;
begin
  rAbZipObject.ErrReset;
  mStream:=TMemoryStream.Create;
  try
    mStream.Write(Buffer^, Length);
    rAbZipObject.AbZip.AddFromStream(NewFile, mStream);
  finally
    mStream.Free;
  end;
  Result:=rAbZipObject.ErrCode;
end;

function TrAbZipObject.AddContent(NewFile: string; Content: PChar): integer;
var
  mStream: TMemoryStream;
  S: TStrings;
begin
  rAbZipObject.ErrReset;
  mStream:=TMemoryStream.Create;
  S:=TStringList.Create;
  try
    S.Text:=Content;
    S.SaveToStream(mStream);
    rAbZipObject.AbZip.AddFromStream(NewFile, mStream);
  finally
    S.Free;
    mStream.Free;
  end;
  Result:=rAbZipObject.ErrCode;
end;

function TrAbZipObject.GetLastErrorMsg: string;
var
  S: string;
begin
  if rAbZipObject.ErrCode in [0 .. 66] then
    rAbZipObject.ErrMessage:=AbStrRes(rAbZipObject.ErrCode);
  S:=rAbZipObject.ErrMessage;
  if rAbZipObject.LastFile <> '' then
    S:=S + ': ' + rAbZipObject.LastFile;
  Result:=PChar(S);
end;

procedure rAbZipObject_Create;
begin
  rAbZipObject:=TrAbZipObject.Create;
  rAbZipObject.AbZip:=TAbZipKit.Create(nil);
  rAbZipObject.AbZip.OnProcessItemFailure:=rAbZipObject.AbZipErr;
  rAbZipObject.AbZip.ExtractOptions:=[eoCreateDirs, eoRestorePath];
  rAbZipObject.AbZip.StoreOptions:=[soReplace];
  rAbZipObject.AbZip.ForceType:=true;
  rAbZipObject.AbZip.ArchiveType:=atZip;
end;

procedure eAbZipObject_Free;
begin
  rAbZipObject.AbZip.Free;
  rAbZipObject.Free;
end;

initialization
  if ModuleIsLib then
    rAbZipObject_Create;

finalization
  if ModuleIsLib then
    eAbZipObject_Free;

end.
